@@ -184,6 +184,8 @@ gupnp_device_info_dispose (GObject *object)
                 g_object_unref (info->priv->doc);
                 info->priv->doc = NULL;
         }
+
+        G_OBJECT_CLASS (gupnp_device_info_parent_class)->dispose (object);
 }
 
 static void
@@ -198,6 +200,8 @@ gupnp_device_info_finalize (GObject *object)
         g_free (info->priv->device_type);
 
         soup_uri_free (info->priv->url_base);
+
+        G_OBJECT_CLASS (gupnp_device_info_parent_class)->finalize (object);
 }
 
 static void
@@ -791,6 +795,9 @@ gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
                 }
         }
 
+        if (icons == NULL)
+                return NULL;
+
         /* Find closest match */
         closest = NULL;
         for (l = icons; l; l = l->next) {
@@ -804,16 +811,20 @@ gupnp_device_info_get_icon_url (GUPnPDeviceInfo *info,
         }
 
         if (!closest) {
-                icon = l->data;
+                for (l = icons; l; l = l->next) {
+                        icon = l->data;
 
-                /* No icons with positive weight, look at ones with
-                 * negative weight */
-                if (!closest || icon->weight > closest->weight)
-                        closest = icon;
+                        /* No icons with positive weight, look at ones with
+                         * negative weight */
+                        if (!closest || icon->weight > closest->weight)
+                                closest = icon;
+                }
         }
 
         /* Fill in return values */
         if (closest) {
+                icon = closest;
+
                 if (mime_type) {
                         if (icon->mime_type) {
                                 *mime_type = g_strdup
