@@ -890,14 +890,14 @@ check_action_response (GUPnPServiceProxy       *proxy,
         case SOUP_STATUS_INTERNAL_SERVER_ERROR:
                 break;
         default:
-                set_server_error (error, action->msg);
+                _gupnp_error_set_server_error (error, action->msg);
 
                 return NULL;
         }
 
         /* Parse response */
-        response = xmlParseMemory (action->msg->response_body->data,
-                                   action->msg->response_body->length);
+        response = xmlRecoverMemory (action->msg->response_body->data,
+                                     action->msg->response_body->length);
 
 	if (!response) {
                 if (action->msg->status_code == SOUP_STATUS_OK) {
@@ -906,7 +906,7 @@ check_action_response (GUPnPServiceProxy       *proxy,
                                      GUPNP_SERVER_ERROR_INVALID_RESPONSE,
                                      "Could not parse SOAP response");
                 } else {
-                        set_error_literal
+                        g_set_error_literal
                                     (error,
                                      GUPNP_SERVER_ERROR,
                                      GUPNP_SERVER_ERROR_INTERNAL_SERVER_ERROR,
@@ -987,10 +987,10 @@ check_action_response (GUPnPServiceProxy       *proxy,
                 if (desc == NULL)
                         desc = g_strdup (action->msg->reason_phrase);
 
-                set_error_literal (error,
-                                   GUPNP_CONTROL_ERROR,
-                                   code,
-                                   desc);
+                g_set_error_literal (error,
+                                     GUPNP_CONTROL_ERROR,
+                                     code,
+                                     desc);
 
                 g_free (desc);
 
@@ -1506,8 +1506,8 @@ server_handler (SoupServer        *soup_server,
                 proxy->priv->seq = 1;
 
         /* Parse the actual XML message content */
-        doc = xmlParseMemory (msg->request_body->data,
-                              msg->request_body->length);
+        doc = xmlRecoverMemory (msg->request_body->data,
+                                msg->request_body->length);
         if (doc == NULL) {
                 /* Failed */
                 g_warning ("Failed to parse NOTIFY message body");
@@ -1602,7 +1602,7 @@ subscription_expire (gpointer user_data)
 
         g_free (sub_url);
 
-        g_assert (msg != NULL);
+        g_return_val_if_fail (msg != NULL, FALSE);
 
         /* Add headers */
         soup_message_headers_append (msg->request_headers,
@@ -1698,7 +1698,7 @@ subscribe_got_response (SoupSession       *session,
 
                         /* We want to resubscribe before the subscription
                          * expires. */
-                        timeout -= GENA_TIMEOUT_DELTA;
+                        timeout = g_random_int_range (1, timeout / 2);
 
                         if (timeout < 0) {
                                 g_warning ("Invalid time-out specified. "
@@ -1725,9 +1725,10 @@ subscribe_got_response (SoupSession       *session,
                 SoupServer *server;
 
                 /* Subscription failed. */
-                error = g_error_new (GUPNP_EVENTING_ERROR,
-                                     GUPNP_EVENTING_ERROR_SUBSCRIPTION_FAILED,
-                                     "%s", msg->reason_phrase);
+                error = g_error_new_literal
+                                (GUPNP_EVENTING_ERROR,
+                                 GUPNP_EVENTING_ERROR_SUBSCRIPTION_FAILED,
+                                 msg->reason_phrase);
 
  ERROR:
                 proxy->priv->subscribed = FALSE;
@@ -1867,17 +1868,17 @@ unsubscribe (GUPnPServiceProxy *proxy)
 
         g_free (sub_url);
 
-        g_assert (msg != NULL);
+        if (msg != NULL) {
+                /* Add headers */
+                soup_message_headers_append (msg->request_headers,
+                                             "SID",
+                                             proxy->priv->sid);
 
-        /* Add headers */
-        soup_message_headers_append (msg->request_headers,
-                                     "SID",
-                                     proxy->priv->sid);
+                /* And queue it */
+                session = gupnp_context_get_session (context);
 
-        /* And queue it */
-        session = gupnp_context_get_session (context);
-
-        soup_session_queue_message (session, msg, NULL, NULL);
+                soup_session_queue_message (session, msg, NULL, NULL);
+        }
 
         /* Reset SID */
         g_free (proxy->priv->sid);
