@@ -69,6 +69,7 @@ xml_doc_wrapper_new (xmlDoc *doc)
         return wrapper;
 }
 
+/* libxml DOM interface helpers */
 xmlNode *
 xml_util_get_element (xmlNode *node,
                       ...)
@@ -97,42 +98,6 @@ xml_util_get_element (xmlNode *node,
         return node;
 }
 
-gboolean
-xml_util_node_get_content_value (xmlNode *node,
-                                 GValue  *value)
-{
-        xmlChar *content;
-        gboolean success;
-
-        content = xmlNodeGetContent (node);
-
-        success = gvalue_util_set_value_from_string (value, (char *) content);
-
-        xmlFree (content);
-
-        return success;
-}
-
-void
-xml_util_set_child_element_content (xmlNode    *node,
-                                    const char *child_name,
-                                    const char *content)
-{
-        xmlNode *child_node;
-
-        child_node = xml_util_get_element (node,
-                                           child_name,
-                                           NULL);
-
-        if (!child_node) {
-                child_node = xmlNewNode (NULL, (xmlChar *) child_name);
-
-                xmlAddChild (node, child_node);
-        }
-
-        xmlNodeSetContent (child_node, (xmlChar *) content);
-}
-
 xmlChar *
 xml_util_get_child_element_content (xmlNode    *node,
                                     const char *child_name)
@@ -184,13 +149,13 @@ xml_util_get_child_element_content_glib (xmlNode    *node,
         return copy;
 }
 
-SoupUri *
+SoupURI *
 xml_util_get_child_element_content_uri (xmlNode    *node,
                                         const char *child_name,
-                                        SoupUri    *base)
+                                        SoupURI    *base)
 {
         xmlChar *content;
-        SoupUri *uri;
+        SoupURI *uri;
 
         content = xml_util_get_child_element_content (node, child_name);
         if (!content)
@@ -206,9 +171,9 @@ xml_util_get_child_element_content_uri (xmlNode    *node,
 char *
 xml_util_get_child_element_content_url (xmlNode    *node,
                                         const char *child_name,
-                                        SoupUri    *base)
+                                        SoupURI    *base)
 {
-        SoupUri *uri;
+        SoupURI *uri;
         char *url;
 
         uri = xml_util_get_child_element_content_uri (node, child_name, base);
@@ -241,3 +206,89 @@ xml_util_get_attribute_contents (xmlNode    *node,
                 return NULL;
 }
 
+/**
+ * xml_util_real_node:
+ * @node: an %xmlNodePtr
+ *
+ * Finds the first "real" node (ie, not a comment or whitespace) at or
+ * after @node at its level in the tree.
+ *
+ * Return: a node, or %NULL
+ *
+ * (Taken from libsoup)
+ **/
+xmlNode *
+xml_util_real_node (xmlNode *node)
+{
+	while (node && (node->type == XML_COMMENT_NODE ||
+			xmlIsBlankNode (node)))
+		node = node->next;
+	return node;
+}
+
+/* XML string creation helpers */
+
+#define INITIAL_XML_STR_SIZE 100 /* Initial xml string size in bytes */
+
+GString *
+xml_util_new_string (void)
+{
+        return g_string_sized_new (INITIAL_XML_STR_SIZE);
+}
+
+void
+xml_util_start_element (GString    *xml_str,
+                        const char *element_name)
+{
+        g_string_append_c (xml_str, '<');
+        g_string_append (xml_str, element_name);
+        g_string_append_c (xml_str, '>');
+}
+
+void
+xml_util_end_element (GString    *xml_str,
+                      const char *element_name)
+{
+        g_string_append (xml_str, "</");
+        g_string_append (xml_str, element_name);
+        g_string_append_c (xml_str, '>');
+}
+
+void
+xml_util_add_content (GString    *xml_str,
+                      const char *content)
+{
+        /* Modified from GLib gmarkup.c */
+        const gchar *p;
+
+        p = content;
+
+        while (*p) {
+                const gchar *next;
+                next = g_utf8_next_char (p);
+
+                switch (*p) {
+                case '&':
+                        g_string_append (xml_str, "&amp;");
+                        break;
+
+                case '<':
+                        g_string_append (xml_str, "&lt;");
+                        break;
+
+                case '>':
+                        g_string_append (xml_str, "&gt;");
+                        break;
+
+                case '"':
+                        g_string_append (xml_str, "&quot;");
+                        break;
+
+                default:
+                        g_string_append_len (xml_str, p, next - p);
+                        break;
+                }
+
+                p = next;
+        }
+}
